/*--------------------------------------------------------------------------*/
/*                                                                          */
/* We have had several customers experience difficulty understanding the    */
/* wn_gets function.  This is understandable, it is somewhat confusing.     */
/* This demo will help you understand the features, including several new   */
/* powerful features, and implement simpler forms to get basic items        */
/* such as time, date, phone numbers, etc.  The modified wn_gets is fully   */
/* compatible with the previous version so none of your code need change.   */
/* However, a new function called wn_gets_ll (which wn_gets calls)          */
/* has added the capability to uppercase the first char of each word,       */
/* strip the mask completely, only strip the end, or not strip at all.      */
/* More importantly, is can allow the input of a string longer than the     */
/* display width of the field and will scroll within this region auto-      */
/* matically!  Is can also display arrows to show the user that the string  */
/* has data to the left or right of the current display field.              */
/*                                                                          */
/*------------------------- flags for wn_gets_ll ---------------------------*/
/* G_STRIP        strips the entire mask from the string.                   */
/*              * 12/04/91 = 120491, (432)-555-3421 = 4325553421						*/
/* G_STRIP_END    strips the end of the string only only.					          */
/*              * Kevin Huck____ = Kevin Huck          											*/
/* G_UP_FST_CHAR  convert the first letter of each word to uppercase        */
/*              * this is a test = This Is A Test                           */
/* G_ARROW        displays left and right arrows if display width is less   */
/*                than field width. NOTE: This requires an additional       */
/*                screen/window space before and after the field.           */
/*--------------------------------------------------------------------------*/
/* Understanding these few flags and looking at the below examples make     */
/* a very complex input routine much more managable!                        */
/*--------------------------------------------------------------------------*/

#include <stdlib.h>
#include "uw.h"
#include "uw_globx.h"
#include "uw_keys.h"

WINDOW Main_wn;

int date_vld(char *w, char *t, char *m, int pos, int max_pos, WINDOW *wnp );
int time_vld(char *w, char *t, char *m, int pos, int max_pos, WINDOW *wnp );

main()
{
	int i, r;
	char name1[32],  name2[32],  name3[32],
			 phone1[16], phone2[16], phone3[16],
			 date1[16],  date2[16],  date3[16],
			 time1[16],  time2[16],  time3[16];
	WINDOW *wnp;

	init_video(80, 25);
	init_clock(0x3333);
	wnp = &Main_wn;
	wn_create( 0, 0, V_cols-1, V_rows-1, DBL_BDR, WN_NORMAL, wnp );
	wn_color( LIGHTGRAY, BLUE, wnp );
	wn_bdr_color( LIGHTGRAY, BLUE, wnp );
	wn_name("<<< EnQue's String Entry Demonstration >>>", wnp);
	wn_set(wnp);

/****************************************************************************/

/*---------- display full width and convert first char to uppercase --------*/
	r = 4;
	msg_line("Entry with first letters capped and no strip");
	wn_plst(  2, r, "Enter your name  : ", wnp );
	my_gets(name1, 22, 22, G_UP_FST_CHAR, wnp);
	wn_plst( 50, r, name1, wnp );

/*------------- display full width and strip the end of the mask -----------*/
	r++;
	msg_line("Entry with strip to end of string");
	wn_plst(  2, r, "Enter your name  : ", wnp );
	my_gets(name2, 22, 22, G_STRIP_END, wnp);
	wn_plst( 50, r, name2, wnp );
/*---------- display only 6 characters, strip the entire mask, -------------*/
/*---------- convert first char, and display arrows...         -------------*/
	r++;
	msg_line("Entry with first letters capped, strip, scrolling, and arrow chars");
	wn_plst(  2, r, "Enter your name  : ", wnp );
	my_gets(name3, 22, 6, G_STRIP | G_UP_FST_CHAR | G_ARROW, wnp);
	wn_plst( 50, r, name3, wnp );

/****************************************************************************/

/*--------------- display full width and strip the entire mask -------------*/
	r += 2;
	msg_line("Entry with strip");
	wn_plst(  2, r, "Enter your phone # : ", wnp );
	my_get_phone(phone1, 14, G_STRIP, wnp);
	wn_plst( 50, r, phone1, wnp );
/*------------------- display full width, don't strip mask -----------------*/
	r++;
	msg_line("Entry without strip");
	wn_plst(  2, r, "Enter your phone # : ", wnp );
	my_get_phone(phone2, 14, G_VALIDATE, wnp);
	wn_plst( 50, r, phone2, wnp );
/*----------------- display 5 chars and strip the entire mask --------------*/
	r++;
	msg_line("Entry with strip and scrolling, but without arrow chars");
	wn_plst(  2, r, "Enter your phone # : ", wnp );
	my_get_phone(phone3, 5, G_STRIP, wnp);
	wn_plst( 50, r, phone3, wnp );

/****************************************************************************/

	set_validation_func( date_vld );
/*--------------- display full width and strip the entire mask -------------*/
	r += 2;
	msg_line("Entry with strip");
	wn_plst(  2, r, "Enter today's date: ", wnp );
	my_get_date(date1, 8, G_STRIP, wnp);
	wn_plst( 50, r, date1, wnp );
	r++;
/*------------------- display full width, don't strip mask -----------------*/
	msg_line("Entry without strip and with validation");
	wn_plst(  2, r, "Enter today's date: ", wnp );
	my_get_date(date2, 8, G_VALIDATE, wnp);
	wn_plst( 50, r, date2, wnp );
	r++;
/*----------------- display 5 chars and strip the entire mask --------------*/
	msg_line("Entry with strip, scrolling, and arrow characters");
	wn_plst(  2, r, "Enter today's date: ", wnp );
	my_get_date(date3, 2, G_STRIP | G_ARROW, wnp);
	wn_plst( 50, r, date3, wnp );

/****************************************************************************/

	set_validation_func( time_vld );
/*--------------- display full width and strip the entire mask -------------*/
	r += 2;
	msg_line("Entry with strip");
	wn_plst(  2, r, "Enter current time: ", wnp );
	my_get_time(time1, 5, G_STRIP, wnp);
	wn_plst( 50, r, time1, wnp );
/*------------------- display full width, don't strip mask -----------------*/
	r++;
	msg_line("Entry without strip and with validation");
	wn_plst(  2, r, "Enter current time: ", wnp );
	my_get_time(time2, 5, G_VALIDATE, wnp);
	wn_plst( 50, r, time2, wnp );
/*----------------- display 5 chars and strip the entire mask --------------*/
	r++;
	msg_line("Entry with strip, scrolling, and arrow characters");
	wn_plst(  2, r, "Enter current time: ", wnp );
	my_get_time(time3, 2, G_STRIP | G_ARROW, wnp);
	wn_plst( 50, r, time3, wnp );
	set_validation_func( NULL );

	msg_line("Demo complete, press any key to quit!");
	wait_event();
	wn_destroy(wnp);
	end_video();
	return(0);
}

/*************/
/* ~msg_line */
/*           ****************************************************************/
/* Display the message at the last line on the screen.                      */
/****************************************************************************/
msg_line( char *msg )
{
	int save_att = Main_wn.att;

	wn_color(BLACK, LIGHTGRAY, &Main_wn);
	mv_cs(0, V_rows - 3, &Main_wn);
	wn_claol(&Main_wn);
	wn_plst(CENTERED, V_rows - 3, msg, &Main_wn);
	Main_wn.att = save_att;
}
/*** end of message_line ***/

my_gets(char *s, int max_len, int disp_width, int flags, WINDOW *wnp)
{
	char mask[81], template[81];

	setmem(s,max_len,0);
	setmem(mask,max_len,'_'), mask[max_len] = '\0';
	setmem(template,max_len,'*'), template[max_len] = '\0';
	return(wn_gets_ll(s, mask, template, swap_nibbles(wnp->att),
		flags, disp_width, wnp));
}

my_get_phone(char *s, int disp_width, int flags, WINDOW *wnp)
{
	char *mask, *template;

	setmem(s,15,0);
	mask     = "(___)-___-____";
	template = " ###  ### ####";
	return(wn_gets_ll(s, mask, template, swap_nibbles(wnp->att),
		flags, disp_width, wnp));
}

my_get_date(char *s, int disp_width, int flags, WINDOW *wnp)
{
	char *mask, *template;

	setmem(s,9,0);
	mask     = "__/__/__";
	template = "## ## ##";
	return(wn_gets_ll(s, mask, template, swap_nibbles(wnp->att),
		flags, disp_width, wnp));
}

my_get_time(char *s, int disp_width, int flags, WINDOW *wnp)
{
	char *mask, *template;

	setmem(s,6,0);
	mask     = "__:__";
	template = "## ##";
	return(wn_gets_ll(s, mask, template, swap_nibbles(wnp->att),
		flags, disp_width, wnp));
}

int date_vld(char *w, char *t, char *m, int pos, int max_pos, WINDOW *wnp )
{
	int ok = 1;

	/* mm/dd/yy */
	if( !range('0',w[0],'1') && (w[0] != m[0]) )
			ok = 0, w[0] = m[0];
	if( w[0] == '0' )
	{
		if( !range('1',w[1],'9') && (w[1] != m[1]) )
				ok = 0, w[1] = m[1];
	}
	else if( w[0] == '1' )
	{
		if( !range('0',w[1],'2') && (w[1] != m[1]) )
				ok = 0, w[1] = m[1];
	}

	if( !range('0',w[3],'3') && (w[3] != m[3]) )
			ok = 0, w[3] = m[3];;
	if( w[3] == '3' )
	{
		if( !range('0',w[4],'1') && (w[4] != m[4]) )
				ok = 0, w[4] = m[4];;
	}else{
		if( !range('0',w[4],'9') && (w[4] != m[4]) )
				ok = 0, w[4] = m[4];;
	}
	if( !range('0',w[6],'9') && (w[6] != m[6]) )
			ok = 0, w[6] = m[6];;
	if( !range('0',w[7],'9') && (w[7] != m[7]) )
			ok = 0, w[7] = m[7];;

	if( !ok )
	{
		tone(1024,20);
		wn_plst( CENTERED, 20, "Invalid Date - Hit any key to continue", wnp );
		wait_event();
		wn_plst( CENTERED, 20, "                                      ", wnp );
		return(0);
	}
	return(1);
}

int time_vld(char *w, char *t, char *m, int pos, int max_pos, WINDOW *wnp )
{
	int ok = 1;

	/* hh:mm */
	if( !range('0',w[0],'2') && (w[0] != m[0]) )
			ok = 0, w[0] = m[0];

	if( w[0] == '2' )
	{
		if( !range('0',w[1],'3') && (w[1] != m[1]) )
				ok = 0, w[1] = m[1];
	}else{
		if( !range('0',w[1],'9') && (w[1] != m[1]) )
				ok = 0, w[1] = m[1];
	}
	if( !range('0',w[3],'5') && (w[3] != m[3]) )
			ok = 0, w[3] = m[3];;
	if( !range('0',w[4],'9') && (w[4] != m[4]) )
			ok = 0, w[4] = m[4];;

	if( !ok )
	{
		tone(1024,20);
		wn_plst( CENTERED, 20, "Invalid Time - Hit any key to continue", wnp );
		wait_event();
		wn_plst( CENTERED, 20, "                                      ", wnp );
		return(0);
	}
	return(1);
}

/**** END OF FILE ****/
